package com.company.jmixexample.api;

import io.jmix.core.DataManager;
import io.jmix.core.JmixSecurityFilterChainOrder;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.web.util.UriTemplate;

import java.io.IOException;

@Configuration
@WebServlet(value = "/internal/builder/{route}", loadOnStartup = 1)
public class BuilderRoute extends HttpServlet {
    protected static final Logger log = LoggerFactory.getLogger(BuilderRoute.class);


    /**
     * Returns a ServletRegistrationBean for the example servlet.
     *
     * @return The ServletRegistrationBean for the example servlet.
     */
    @Bean
    public ServletRegistrationBean builderServletBean() {
        ServletRegistrationBean bean = new ServletRegistrationBean(
                this, "/internal/builder/*");
        bean.setLoadOnStartup(1);
        return bean;
    }

    @Bean(name = "builderSecurityFilterChain")
    @Order(JmixSecurityFilterChainOrder.FLOWUI - 10)
    // https://forum.jmix.io/t/static-resources-problem/2351/10
    // Allow HTTP Request without Session (Not WORKING !!), i want Save (POST) Action via Session and View (GET) Action without Session.
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http.securityMatcher("/internal/builder/**")
                .authorizeHttpRequests((authorize) -> authorize.requestMatchers("/internal/builder/**").permitAll());
        return http.build();
    }



    /**
     * Handles a POST request and saves the page content to the database.
     *
     * @param requestIO  The HttpServletRequest object.
     * @param responseIO The HttpServletResponse object.
     * @throws ServletException If an error occurs during the request processing.
     * @throws IOException      If an I/O error occurs while handling the response.
     */
    @Override
    protected void doPost(HttpServletRequest requestIO, HttpServletResponse responseIO) throws ServletException, IOException {
        // Parse Route from Request.
        String routeIO  = extractRouteFromUri(requestIO.getRequestURI());

        // Hardcoded Example:
        // Check if User is logged in and has Permission to save Page.
        if(true) {
            // Check if Page is existing in Database.
            if(true) {
                // Save Page to Database (Not implemented for Example).
                // For Testing purposes only a Example Response.
                responseIO.getWriter().write("{\"status\":\"OK\"}");
            }  else {
                // Write Error wich Builder understands.
                // For Testing purposes only a Example Response.
                responseIO.getWriter().write("{\"status\":\"ERROR\"}");
            }
        } else {
            // Write Error wich Builder understands.
            // For Testing purposes only a Example Response.
            responseIO.getWriter().write("{\"status\":\"ERROR\"}");
        }

    }

    /**
     * Handles a GET request and retrieves the page content based on the requested route.
     *
     * @param requestIO  The HttpServletRequest object.
     * @param responseIO The HttpServletResponse object.
     * @throws ServletException If an error occurs during the request processing.
     * @throws IOException      If an I/O error occurs while handling the response.
     */
    @Override
    protected void doGet(HttpServletRequest requestIO, HttpServletResponse responseIO) throws ServletException, IOException {
        // Parse Route from Request.
        String routeIO = extractRouteFromUri(requestIO.getRequestURI());

        // Hardcoded Example:
        if(routeIO.equals("listPages")) {
            // For Testing purposes only a Example Response.
            // Respond with Pages Entities from Database.
            responseIO.getWriter().write("{\"status\":\"OK\", \"pages\": []}");
        } else {
            // For Testing purposes only a Example Response.
            // Write given Page to Client (if exists).
            responseIO.getWriter().write("<html><head></head></body><h1>Hello from JMIX CMS :)</h1></body></html");
        }
    }

    /**
     * Extracts the route from the given URI using the provided UriTemplate.
     *
     * @param uri The URI from which to extract the route.
     * @return The extracted route.
     */
    private String extractRouteFromUri(String uri) {
        UriTemplate template = new UriTemplate("/internal/builder/{route}");
        return template.match(uri).get("route");
    }
}
