package com.company.jmixexample.api;

import io.jmix.core.JmixSecurityFilterChainOrder;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.web.util.UriTemplate;

import java.io.IOException;

@Configuration
@WebServlet(value = "/internal/payment/{provider}", loadOnStartup = 1)
public class PaymentRoute extends HttpServlet {
    protected static final Logger log = org.slf4j.LoggerFactory.getLogger(PaymentRoute.class);

    @Bean(name = "paymentSecurityFilterChain")
    @Order(JmixSecurityFilterChainOrder.FLOWUI - 20)
    // https://forum.jmix.io/t/static-resources-problem/2351/10
    // Allow HTTP Request without Session (Not WORKING !!), but i want that i need no valid Session.
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http.securityMatcher("/internal/payment/**").authorizeHttpRequests((authorize) -> {
            authorize.requestMatchers("/internal/payment/**").permitAll();
        });

        return http.build();
    }

    @Bean
    public ServletRegistrationBean paymentServletBean() {
        ServletRegistrationBean bean = new ServletRegistrationBean(
                this, "/internal/payment/*");
        bean.setLoadOnStartup(1);
        return bean;
    }

    /**
     * Handles a POST request to authorize a payment with a specified provider.
     *
     * @param requestIO  the HTTP servlet request object
     * @param responseIO the HTTP servlet response object
     * @throws ServletException if a servlet-specific error occurs during this request
     * @throws IOException      if an I/O error occurs during this request
     */
    @Override
    protected void doPost(HttpServletRequest requestIO, HttpServletResponse responseIO) throws ServletException, IOException {
        this.handleRequest(requestIO, responseIO);
    }

    /**
     * Handles a request for payment authorization with a specified provider.
     *
     * @param requestIO  the HTTP servlet request object
     * @param responseIO the HTTP servlet response object
     */
    private void handleRequest(HttpServletRequest requestIO, HttpServletResponse responseIO) throws IOException {
        String providerIO = this.extractProviderFromUri(requestIO.getRequestURI());

        // 1. Get Extension Handler for IPN Handling (PayPal.jar -> IPNHandler -> handle());
        // IF PROVIDER NOT EXISTS then ERROR
        // IF PROVIDER EXISTS then handle Request and Update JMIX Entity.

        // Hardcoded Example:
        if(providerIO.equals("paypal")) {
            // Handle IPN Request (Not implemented in Example).
            // ExtensionHandler.get(IPNHandler.class).byName("PayPal").handle(requestIO, responseIO);

            // For Testing purposes only a Example Response.
            responseIO.getWriter().write("{\"status\":\"OK\"}");
        } else {
            // Write Error wich IPN understands...
            // For Testing purposes only a Example Response.
            responseIO.getWriter().write("{\"status\":\"ERROR\"}");
        }
    }

    /**
     * Handles a GET request to authorize a payment with a specified provider.
     *
     * @param requestIO  the HTTP servlet request object
     * @param responseIO the HTTP servlet response object
     * @throws ServletException if a servlet-specific error occurs during this request
     */
    @Override
    protected void doGet(HttpServletRequest requestIO, HttpServletResponse responseIO) throws ServletException, IOException {
        this.handleRequest(requestIO, responseIO);
    }



    /**
     * Extracts the provider name from the given URI.
     *
     * @param uri The URI string from which to get the provider name
     * @return The provider name extracted from the URI
     */
    private String extractProviderFromUri(String uri) {
        UriTemplate template = new UriTemplate("/internal/payment/{provider}");
        return template.match(uri).get("provider");
    }
}
